using System;
using System.Data.SqlClient;
using System.Configuration;
using System.Web;
using SSK.Client.License;
using System.Xml;
using Microsoft.ApplicationBlocks.Data;

namespace SSK.Server 
{

/// <summary>
/// Implements all the database processing and access logic.
/// </summary>
	public class DBHelper
	{
		private SqlConnection conn;

		public DBHelper()
		{

		}

		/// <summary>
		/// Ensures that a string that will be used for a SQL command
		/// does not contain invalid characters
		/// </summary>
		/// <param name="string"></param>
		public void ValidateArgForSQL(ref string arg)
		{
			string validchars = " ABCDEFGHIJKLMNOPQRSTUVWXYZ" +
				"abcdefghijklmnopqrstuvwxyz@" +
				"01234567890:~!@#$%^&*()_?-+={}[]|" +
				"/" + "\\";

			// Go through the string and vefify that each
			// character exists in validchars
			int len = arg.Length;
			int i = 0;
			char bad = ' ';

			for (i = 0; i < len; i++)
			{
				if (validchars.IndexOf(arg[i]) == -1)
				{
					bad = arg[i];

					arg = arg.Remove(i, 1);
					i -= 1;
					len = arg.Length;
				}
			}

			if ((i = arg.IndexOf("--")) != -1)
			{
				arg = arg.Remove(i, 2);
			}

		}


		/// <summary>
		/// Ensures that a string that will be used for a SQL command
		/// does not contain invalid characters
		/// </summary>
		/// <param name="arg"></param>
		public void ValidateArgForSQL(ref DateTime arg)
		{
			DateTime.Parse(arg.ToString());
		}

		/// <summary>
		/// Insert a record into the SSK_Issue table
		/// </summary>
		/// <param name="productName">Product Name</param>
		/// <param name="licenseLevel">License Level</param>
		/// <param name="emailContact">Email contact (name@server.com)</param>
		/// <param name="firstName">First Name</param>
		/// <param name="lastName">Last Name</param>
		/// <param name="description">Description of issue</param>
		public int SubmitIssue(string productName,
			string licenseLevel,
			string emailContact,
			string firstName,
			string lastName,
			string description,
            string devID,
            string uniqueMachineID)
		{


			return(Convert.ToInt32(SqlHelper.ExecuteScalar(ConfigurationManager.AppSettings["SQLConnectionString"],
									"SubmitIssue",
									productName,
									licenseLevel,
									emailContact,
									firstName,
									lastName,
									description,
                                    devID,
                                    uniqueMachineID)));
		}

        /// <summary>
        /// Return a filled ClientAppConfig class
        /// </summary>
        /// <returns></returns>
        public ClientAppConfig FillClientAppConfig
        (
            string productName, 
            string devToken, 
            string uniqueMachineID, 
            DateTime startDateTime,
            string Url
        )
        {
            try
            {
                ClientAppConfig clientAppConfig = new ClientAppConfig();

                #region Get the client configuration settings
                //Get the client configuration settings
                SqlDataReader reader = SqlHelper.ExecuteReader(ConfigurationManager.AppSettings["SQLConnectionString"],
                            "GetProductConfigSettings", productName, devToken);

                if (reader.Read())
                {
                    clientAppConfig.BuyNowMaxNumPoll = Convert.ToInt32(reader["BuyNowMaxNumPoll"]);
                    clientAppConfig.ActivateNowMaxNumPoll = Convert.ToInt32(reader["ActivateNowMaxNumPoll"]);
                    clientAppConfig.TrialLengthDays = Convert.ToInt32(reader["TrialLengthDays"]);
                    clientAppConfig.AbortTimerLength = Convert.ToInt32(reader["AbortTimerLength"]);
                    clientAppConfig.PollingTimerLength = Convert.ToInt32(reader["PollingTimerLength"]);
                    
                    
                    
                }
                else
                {
                    throw new SharewareServerException("No configuration data was found for this application.");
                }
               
                reader.Close();
                #endregion

                #region Get the product license levels and prices
                LicenseLevelsPrice[] licenseLevelsPrices;
                LicenseLevelsPrice licenseLevelPrice;



                reader = SqlHelper.ExecuteReader(ConfigurationManager.AppSettings["SQLConnectionString"],
                                            "GetProductLicenseLevels", productName, devToken);
                int numRows = 0;
                while (reader.Read())
                {
                    numRows++;
                }

                licenseLevelsPrices = new LicenseLevelsPrice[numRows];
                reader.Close();

                reader = SqlHelper.ExecuteReader(ConfigurationManager.AppSettings["SQLConnectionString"],
                            "GetProductLicenseLevels", productName, devToken);
                
                int i = 0;
                while (reader.Read())
                {
                    licenseLevelPrice = new LicenseLevelsPrice();

                    licenseLevelPrice.LicenseLevel = Convert.ToString(reader["LicenseLevel"]);
                    licenseLevelPrice.LicenseLevelPrice = Convert.ToDouble(reader["LicenseLevelPrice"]);
                    licenseLevelPrice.IsTrialLicensekey = Convert.ToBoolean(reader["TrialLicenseKey"]);
                    licenseLevelPrice.LicenseLevelID = Convert.ToInt32(reader["ClientID"]);
                    licenseLevelPrice.Description = Convert.ToString(reader["Description"]);

                    licenseLevelsPrices[i] = licenseLevelPrice;
                    i++;
                    
                }
                clientAppConfig.LicenseLevelPrices = licenseLevelsPrices;
                reader.Close();
                #endregion

                #region Get the product payment settings
                reader = SqlHelper.ExecuteReader(ConfigurationManager.AppSettings["SQLConnectionString"],
                                            "GetProductPaymentSettings",  devToken);

                PaymentSettings paymentSettings = new PaymentSettings();
                if (reader.Read())
                {

                    paymentSettings.PayPalAccountEmail = Convert.ToString(reader["PayPal_AccountEmail"]);
                    paymentSettings.PayPalItemNumber = Convert.ToString(reader["PayPal_ItemNumber"]);
                    paymentSettings.PayPalQuantity = Convert.ToInt32(reader["PayPal_Quantity"]);
                    paymentSettings.PayPalCurrencyCode = Convert.ToString(reader["PayPal_currencyCode"]);
                    paymentSettings.PayPalNotifyURL = Convert.ToString(reader["PayPal_notifyURL"]);
                    paymentSettings.PayPalRequireShippingAddress = Convert.ToBoolean(reader["PayPal_requireShippingAddress"]);
                    paymentSettings.PayPalIncludeNote = Convert.ToBoolean(reader["PayPal_includeNote"]);
                    paymentSettings.PayPalLogoURL = Convert.ToString(reader["PayPal_logoURL"]);
                    paymentSettings.PayPalTag = Convert.ToString(reader["PayPal_tag"]);
                    paymentSettings.PayPalNoteTitle = Convert.ToString(reader["PayPal_noteTitle"]);
                    paymentSettings.PayPalSuccessURL = Convert.ToString(reader["PayPal_successUrl"]);
                    paymentSettings.PayPalCancelURL= Convert.ToString(reader["PayPal_cancelURL"]);
                    paymentSettings.PaymentURL = Convert.ToString(reader["Payment_URL"]);


                    if (paymentSettings.PaymentURL == "")
                   {
                       
                        paymentSettings.PaymentURL = Url.Substring(0, Url.ToLower().IndexOf("$safeprojectname$.asmx".ToLower())) + "testpaymentserver.aspx";
                    }


                    clientAppConfig.PaymentSettings = paymentSettings;
                }
                reader.Close();
                #endregion

                #region Get the product data store settings

                string serverPublicKey;
                LicenseServer server = new LicenseServer();
                server.CreateNewProduct(productName, out serverPublicKey);
                clientAppConfig.ServerPublicKey = serverPublicKey;
                clientAppConfig.DevToken = devToken;
                clientAppConfig.ProductName = productName;

                reader = SqlHelper.ExecuteReader(ConfigurationManager.AppSettings["SQLConnectionString"],
                                            "GetProductDataStoreSettings", devToken, productName, uniqueMachineID);

                DataStore dataStoreSettings = new DataStore();
                if (reader.Read())
                {

                    dataStoreSettings.DateTime = Convert.ToDateTime(reader["DateTime"]);
                    dataStoreSettings.LicenseLevel = Convert.ToString(reader["LicenseLevel"]);
                    dataStoreSettings.NumberOfTrialDaysElapsed = Convert.ToInt32(reader["NumberOfTrialDaysElapsed"]);
                    dataStoreSettings.StartDate = Convert.ToDateTime(reader["StartDate"]);
                    dataStoreSettings.UserInfoRegistered = Convert.ToBoolean(reader["UserInfoRegistered"]);
                    dataStoreSettings.UniqueMachineID = Convert.ToString(reader["UniqueMachineID"]);
                    dataStoreSettings.RegistrationKey = Convert.ToString(reader["RegistrationKey"]);
                    

                    clientAppConfig.DataStoreSettings = dataStoreSettings;
                }
                //there is not an entry for the datastore so create a new one
                else
                {

                    dataStoreSettings.DateTime = DateTime.Now;
                    dataStoreSettings.NumberOfTrialDaysElapsed = 0;
					dataStoreSettings.StartDate = startDateTime;
                    dataStoreSettings.UserInfoRegistered = false;
                    dataStoreSettings.UniqueMachineID = uniqueMachineID;

                    foreach (LicenseLevelsPrice trialLicenseLevel in licenseLevelsPrices)
                    {
                        if (trialLicenseLevel.IsTrialLicensekey)
                        {
                            dataStoreSettings.LicenseLevel = trialLicenseLevel.LicenseLevel;
                        }
                    }

                    LicenseRequest licenseRequest = new LicenseRequest();


                    LicenseClient licenseClient = new LicenseClient(productName, serverPublicKey, uniqueMachineID);

                    licenseRequest = licenseClient.CreateLicenseRequest(dataStoreSettings.LicenseLevel);



                    // Build a LicenseResponse for the client since the invoice is valid.
                    LicenseResponse response = server.CreateLicenseReponse(licenseRequest);
                    dataStoreSettings.RegistrationKey = response.GetAuthenticatedLicenseAsString();
                    SqlHelper.ExecuteNonQuery(ConfigurationManager.AppSettings["SQLConnectionString"],
                                             "InitializeProductDataStoreSettings", devToken, dataStoreSettings.DateTime,
                                             productName,
                                             dataStoreSettings.LicenseLevel,
                                             dataStoreSettings.NumberOfTrialDaysElapsed,
                                             dataStoreSettings.RegistrationKey,
                                             dataStoreSettings.StartDate,
                                             dataStoreSettings.UserInfoRegistered,
                                             dataStoreSettings.UniqueMachineID);

                    dataStoreSettings.RefreshDataStore = false;

                    clientAppConfig.DataStoreSettings = dataStoreSettings;

                }
                reader.Close();
                #endregion

                return clientAppConfig;
            }
            catch (Exception ex)
            {
                throw new SharewareServerException("There was an error attempting to get the client configuration info.", ex);
            }

        }

		/// <summary>
		/// insert a record into the SSK_ProductException table
		/// </summary>
		/// <param name="message">The exception message.</param>
		/// <param name="source">The source of the exception.</param>
		/// <param name="stackTrace">The stacktrace of the exception.</param>
		/// <param name="methodName">The methodName that threw the exception.</param>
		/// <param name="productName">THe productName that threw the exception.</param>
		public int SubmitException(string message,
			string source,
			string stackTrace,
			string methodName,
			string productName,
            string devToken,
            string licenseLevel,
            string uniqueMachineID)
		{


            return Convert.ToInt32(SqlHelper.ExecuteScalar(ConfigurationManager.AppSettings["SQLConnectionString"],
						"SubmitException",
						message,
						source,
						stackTrace,
						methodName,
						productName,
                        devToken,
                        licenseLevel,
                        uniqueMachineID));
		}

		/// <summary>
		/// Inserts a record into the SSK_User table
		/// </summary>
		/// <param name="lastname">Last Name</param>
		/// <param name="firstname">First Name</param>
		/// <param name="address">Address</param>
		/// <param name="city">City</param>
		/// <param name="state">State</param>
		/// <param name="zip">Zip code</param>
		/// <param name="phone">Phone Number ((###) ###-####)</param>
		/// <param name="email">Email contact (name@server.com)</param>
		public int AddUser(string lastname,
			string firstname,
			string address,
			string city,
			string state,
			string zip,
			string phone,
			string email,
            string devToken,
            string productName,
            string licenseLevel,
            string uniqueMachineID)
		{

            return (Convert.ToInt32(SqlHelper.ExecuteScalar(ConfigurationManager.AppSettings["SQLConnectionString"],
				"AddUserInfo",
				lastname,
				firstname,
				address,
				city,
				state,
				zip,
				phone,
				email,
                devToken,
                productName,
                licenseLevel,
                uniqueMachineID)));

		}

		/// <summary>
		/// Adds a record to the SSK_License_Transaction table.
		/// </summary>
		/// <param name="clientIdentity">String that uniquely identifies the client</param>
		/// <param name="licenseLevel">License Level</param>
		/// <param name="product">Product Name</param>
		/// <param name="invoice">Invoice ID given at the creation of a LicenseRequest
		/// by the client</param>
		public void StoreLicenseTransaction(string clientIdentity,
			string licenseLevel,
			string product,
			string invoice,
            string devID,
            string clientSerialId)
		{

            SqlHelper.ExecuteNonQuery(ConfigurationManager.AppSettings["SQLConnectionString"],
										"StoreLicenseTransaction",
										invoice,
										clientIdentity,
										product,
										licenseLevel,
                                        devID,
                                        clientSerialId);

		}

		/// <summary>
		/// Marks an invoice in the SSK_coupon table with the current date/time.
		/// </summary>
		/// <param name="invoice"></param>
		public void SetCouponDate(string invoice)
		{

            SqlHelper.ExecuteNonQuery(ConfigurationManager.AppSettings["SQLConnectionString"],
							"SetCouponDate",
							invoice);
		}

		/// <summary>
		/// Adds (or stores) the authLicense string to the record in the SSK_License_Transaction table
		/// that matches both clientIdentity and invoice
		/// </summary>
		/// <param name="clientIdentity">Client application identifier</param>
		/// <param name="invoice">Invoice ID generated by the client</param>
		/// <param name="authLicense">License string generated by the server which is returned to the 
		/// client</param>
		public void StoreAuthLicense(string clientIdentity,
			string invoice,	string authLicense)
		{

            SqlHelper.ExecuteNonQuery(ConfigurationManager.AppSettings["SQLConnectionString"],
				"StoreAuthLicense",
				invoice,
				authLicense);


		}

        /// <summary>
        /// Update the SSK_Datastore table to reflect the product registration
        /// </summary>
        /// <param name="devToken">The unique identifier for the developer.</param>
        /// <param name="productName">The unique name for this product.</param>
        /// <param name="uniqueMachineID">The unique ID for the client machine</param>
        public int UpdateDataStoreRegistration(string devToken, string productName, string uniqueMachineID, Boolean isRegistered)
        {
            return (Convert.ToInt32(SqlHelper.ExecuteScalar(ConfigurationManager.AppSettings["SQLConnectionString"],
                "UpdateDataStoreRegistration",
                devToken,
                productName,
                uniqueMachineID,
                isRegistered)));
        }


        /// <summary>
        /// Update the SSK_Datastore table to reflect the successful product key approval 
        /// </summary>
        /// <param name="devToken">The unique identifier for the developer.</param>
        /// <param name="productName">The unique name for this product.</param>
        /// <param name="uniqueMachineID">The unique ID for the client machine</param>
        /// <param name="productKey">The authorized product key for the user</param>
        /// <param name="licenseLevel">The requested license level for the product</param>
        public void UpdateDataStoreRegistrationKey(string devToken, string productName, string uniqueMachineID, string registrationKey, string licenseLevel)
        {
            SqlHelper.ExecuteNonQuery(ConfigurationManager.AppSettings["SQLConnectionString"],
                "UpdateDataStoreRegistrationKey",
                devToken,
                productName,
                uniqueMachineID,
                registrationKey,
                licenseLevel);
        }

		/// <summary>
		/// Marks a record in the SSK_License_Transaction table with a string.
		/// </summary>
		/// <param name="invoice">Invoice ID generated by the client</param>
		/// <param name="result"></param>
		public void MarkInvoice(string invoice, string result)
		{
            SqlHelper.ExecuteNonQuery(ConfigurationManager.AppSettings["SQLConnectionString"],
				"MarkInvoice",
				invoice,
				result);
		}

		
		/// <summary>
		/// InvoiceInPaypalTransaction returns true if the invoice argument
		/// is found in a transaction inside the Paypal_Transaction table.  
		/// Returns false if it's not found in the Paypal_Transaction table.
		/// </summary>
		/// <param name="invoice">Invoice ID generated by the client</param>
		/// <returns>true - invoice found in Paypal_Transaction table record</returns>
		public bool InvoiceInPaypalTransaction(string invoice)
		{
			bool inPaypal = false;
            Logger.Log("GetLicense : InvoiceInPaypalTransactipn : Invoice = " + invoice);
            SqlDataReader reader = SqlHelper.ExecuteReader(ConfigurationManager.AppSettings["SQLConnectionString"],
									"InvoiceInPaymentTransaction", invoice);

            if (reader.HasRows)
            {
                inPaypal = true;
                reader.Read();

                string paymentStatus = (string)reader["PaymentStatus"];
                string pendingReason = (string)reader["PendingReason"];

                switch (paymentStatus)
                {
                    case "Completed":
                        return true;

                        break;
                    case "Pending":

                        switch (pendingReason)
                        {
                            case "Echeck":
                                throw new PaypalException(
                                    ConfigurationManager.AppSettings["EcheckMessage"]);


                            case "Multi-Currency":
                                throw new PaypalException(
                                    ConfigurationManager.AppSettings["Problem"].Replace(
                                    "*invoice*", invoice)
                                    );


                            case "Intl":
                                throw new PaypalException(
                                    ConfigurationManager.AppSettings["Problem"].Replace(
                                    "*invoice*", invoice)
                                    );


                            case "Verify":
                                throw new PaypalException(
                                    ConfigurationManager.AppSettings["Problem"].Replace(
                                    "*invoice*", invoice)
                                    );


                            case "Address":
                                throw new PaypalException(
                                    ConfigurationManager.AppSettings["Problem"].Replace(
                                    "*invoice*", invoice)
                                    );


                            case "Upgrade":
                                throw new PaypalException(
                                    ConfigurationManager.AppSettings["Problem"].Replace(
                                    "*invoice*", invoice)
                                    );


                            case "Unilateral":
                                throw new PaypalException(
                                    ConfigurationManager.AppSettings["Problem"].Replace(
                                    "*invoice*", invoice)
                                    );


                            case "Other":
                                throw new PaypalException(
                                    ConfigurationManager.AppSettings["Problem"].Replace(
                                    "*invoice*", invoice)
                                    );


                        }

                        break;
                    case "Failed":
                        throw new PaypalException(
                            ConfigurationManager.AppSettings["Problem"].Replace(
                            "*invoice*", invoice)
                            );

                        break;
                    case "Denied":
                        throw new PaypalException(
                            ConfigurationManager.AppSettings["Problem"].Replace(
                            "*invoice*", invoice)
                            );

                        break;
                    case "Refunded":
                        inPaypal = false;
                        break;
                    case "Cancelled":
                        inPaypal = false;
                        break;
                    default:
                        inPaypal = false;
                        break;
                }

            }
            else
            {
                Logger.Log("GetLicense : InvoiceInPaypalTransactipn : Reader has no rows ");
            }
                Logger.Log("GetLicense : InvoiceInPaypalTransactipn : Completed");
			return inPaypal;
		}

		/// <summary>
		/// Given the invoice number, this function needs to run a query
		/// on the database that figures out if a license can be returned
		/// based on 1)how many licenses it already has given and 2) the 
		/// number of licenses it can give out based on the price
		/// of the paypal transaction.
		/// </summary>
		/// <param name="invoice"></param>
		/// <returns></returns>
		public bool AvailableLicenseForInvoice(string invoice)
		{
			int approvedInvoiceCount = 0;
			int allowedInvoiceCount = 0;

            //try to get the invoice counts.  If we except then
            //catch and set values to 0 for each
            try
            {
                approvedInvoiceCount = (int)SqlHelper.ExecuteScalar(ConfigurationManager.AppSettings["SQLConnectionString"],
                                        "GetApprovedInvoiceCount", invoice);

                allowedInvoiceCount = (int)SqlHelper.ExecuteScalar(ConfigurationManager.AppSettings["SQLConnectionString"],
                                        "GetAllowedInvoiceCount", invoice);
            }
            catch
            {
                approvedInvoiceCount = 0;
                allowedInvoiceCount = 0;
            }

            if (approvedInvoiceCount < allowedInvoiceCount)
				{
					return true;
				}
				else
				{
					return false;
				}

		}

		/// <summary>
		/// Returns whether the Invoice ID is registered in the SSK_Coupon_Number table
		/// </summary>
		/// <param name="invoice">Invoice ID generated by the client</param>
        /// <param name="licenseLevel">The license level for the requested activation.</param>
		/// <returns>true if the invoice is found in the SSK_Coupon_Number table</returns>
		public bool IsInvoiceACoupon(string invoice, ref string licenseLevel, string devToken)
		{
			bool isCouponListed = false;

			// Check to see if the invoice number is a record in the coupon
			// table.

            SqlDataReader reader = SqlHelper.ExecuteReader(ConfigurationManager.AppSettings["SQLConnectionString"],
						"IsInvoiceACoupon", invoice, devToken);

			if (reader.Read())
			{
               
                if (Convert.ToString(reader["used_date"]) == "")
                {
                    isCouponListed = true;
                    SetCouponDate(invoice);
                    licenseLevel = Convert.ToString(reader["license_level"]);
                    if (licenseLevel == "" || licenseLevel == null)
                    {
                        throw new NoAvailableLicensesException(
                            "Reason: 'There was a problem determining the correct product license level. " +
                            " There was no license level assigned to the activation key.'");
                    }
                }
                else
                {
                    //if the used date is null then we must have failed because of a license
                    //level request mismatch
                    if (Convert.ToString(reader["used_date"]) == "")
                    {
                        throw new NoAvailableLicensesException(
                            "Reason: 'The requested license level does not match the license " +
                            "level reserved for this product activation key.'");
                    }
                    else
                    {
                        throw new NoAvailableLicensesException(
                            "Reason: 'The product activation key has already been used " +
                            "or does not exist in the system.  Please " +
                            "contact customer support.'");
                    }

				}
               
			}



			return isCouponListed;
		}

		/// <summary>
		/// VerifyLicenseAvailableInDB is the method that determines if either 
		/// an Invoice ID is available inside the Paypal_transaction table
		/// or the SSK_Coupon_Number table
		/// </summary>
		/// <param name="clientIdentity"></param>
		/// <param name="licenseLevel"></param>
		/// <param name="product"></param>
		/// <param name="invoice"></param>
		/// <returns></returns>
		public bool VerifyLicenseAvailableInDB(string clientIdentity,
			ref string licenseLevel,
			string product,
			string invoice,
            string devToken)
		{
			bool ret = false;

			// Is the invoice number present in the PaypalTransaction
			// table with a successful authorization?  If so, give the license
			// if available.
			if (InvoiceInPaypalTransaction(invoice))
			{
                Logger.Log("VerifyLicenseAvailableInDB : InvoiceInPaypaltransaction : True");

				// Now that we know the invoice number is an official
				// paypal transaction that was paid, we need to figure out 
				// if this invoice number has exceeded or has available 
				// licenses.  This check is done in the SSK_Product table

				// Question: Given Invoice number and price paid (get from
				// Paypal table and join to SSK_Product table with ProductName)
				// , how many licenses do i get from the SSK_Product table?
				if (AvailableLicenseForInvoice(invoice))
				{
					// This function already incremented the licenses
					// for this invoice, so just setup for returning
					// true such that the web service can give the user
					// a license request.
					ret = true;
					MarkInvoice(invoice, "approved");
				}
				else
				{
                    throw new PaypalException(
                        "Reason: 'The approved InvoiceID used for this transaction " +
                        "has exceeded the allowed number of licenses.  Please " +
                        "register a new InvoiceID or contact customer support.'");
                }

            }
			else
			{

                Logger.Log("VerifyLicenseAvailableInDB : InvoiceInPaypaltransaction : False");

				// Invoice was not in the PaypalTransaction table, check
				// to see if it's in the Coupon table
				if (IsInvoiceACoupon(invoice, ref licenseLevel, devToken))
				{
					// if it finds an available coupon that has the same
					// invoice code, it should set the useddate on the coupon.
					ret = true;
					MarkInvoice(invoice, "approved");
				}

			}

			return ret;
		}
	}
}